import 'package:flutter/material.dart';

enum CustomAlertDialogType {
  success,
  error,
  warning,
  info,
}

Future<T?> showCustomAlertDialog<T>({
  required BuildContext context,
  required CustomAlertDialogType type,
  required String title,
  required String content,
}) {
  return showDialog<T>(
    context: context,
    builder: (BuildContext context) {
      return CustomAlertDialog(
        type: type,
        title: title,
        content: content,
      );
    },
  );
}

Future<SnackBarClosedReason> showSuccessDialog<T>({
  required BuildContext context,
  required String title,
  required String message,
  Duration duration = const Duration(seconds: 3),
}) {
  final x = ScaffoldMessenger.of(context).showSnackBar(
    SnackBar(
      padding: const EdgeInsets.all(16),
      backgroundColor: Theme.of(context).backgroundColor,
      // backgroundColor: Colors.transparent,
      content: CustomAlertDialog(
        title: title,
        content: message,
        type: CustomAlertDialogType.success,
        onClick: () => ScaffoldMessenger.of(context).hideCurrentSnackBar(),
      ),
      duration: duration,
    ),
  );

  return x.closed;
}

Future<SnackBarClosedReason> showErrorDialog<T>({
  required BuildContext context,
  required String title,
  required String message,
  Duration duration = const Duration(seconds: 3),
}) {
  final x = ScaffoldMessenger.of(context).showSnackBar(
    SnackBar(
      padding: const EdgeInsets.all(16),
      backgroundColor: Theme.of(context).backgroundColor,
      // backgroundColor: Colors.transparent,
      content: CustomAlertDialog(
        title: title,
        content: message,
        type: CustomAlertDialogType.error,
        onClick: () => ScaffoldMessenger.of(context).hideCurrentSnackBar(),
      ),
      duration: duration,
    ),
  );

  return x.closed;
}

Future<SnackBarClosedReason> showWarningDialog<T>({
  required BuildContext context,
  required String title,
  required String message,
  Duration duration = const Duration(seconds: 3),
}) {
  final x = ScaffoldMessenger.of(context).showSnackBar(
    SnackBar(
      padding: const EdgeInsets.all(16),
      backgroundColor: Theme.of(context).backgroundColor,
      // backgroundColor: Colors.transparent,
      content: CustomAlertDialog(
        title: title,
        content: message,
        type: CustomAlertDialogType.warning,
        onClick: () => ScaffoldMessenger.of(context).hideCurrentSnackBar(),
      ),
      duration: duration,
    ),
  );

  return x.closed;
}

Future<SnackBarClosedReason> showInfoDialog<T>({
  required BuildContext context,
  required String title,
  required String message,
  Duration duration = const Duration(seconds: 3),
}) {
  final x = ScaffoldMessenger.of(context).showSnackBar(
    SnackBar(
      padding: const EdgeInsets.all(16),
      backgroundColor: Theme.of(context).backgroundColor,
      // backgroundColor: Colors.transparent,
      content: CustomAlertDialog(
        title: title,
        content: message,
        type: CustomAlertDialogType.info,
        onClick: () => ScaffoldMessenger.of(context).hideCurrentSnackBar(),
      ),
      duration: duration,
    ),
  );

  return x.closed;
}

class CustomAlertDialog extends StatelessWidget {
  final CustomAlertDialogType type;
  final String title;
  final String content;
  final Widget? icon;
  final String buttonLabel;
  final TextStyle? titleStyle;
  final TextStyle? contentStyle;
  final VoidCallback? onClick;

  const CustomAlertDialog({
    Key? key,
    required this.title,
    required this.content,
    this.icon,
    this.type = CustomAlertDialogType.info,
    this.buttonLabel = "Ok",
    this.onClick,
    this.titleStyle,
    this.contentStyle,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Material(
      child: Padding(
        padding: const EdgeInsets.only(top: 8, bottom: 8),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            icon ??
                Icon(
                  _getIconForType(type),
                  color: getColorForType(type),
                  size: 50,
                ),
            const SizedBox(height: 8.0),
            Text(
              title,
              style: titleStyle ?? Theme.of(context).textTheme.caption,
              textAlign: TextAlign.center,
            ),
            const SizedBox(height: 8.0),
            Text(
              content,
              softWrap: true,
              textAlign: TextAlign.center,
              style: contentStyle ?? Theme.of(context).textTheme.subtitle1,
            ),
            const SizedBox(height: 8.0),
            SizedBox(
              width: double.infinity,
              child: Container(
                padding: const EdgeInsets.all(8.0),
                child: MaterialButton(
                  child: Text(
                    buttonLabel,
                    style: Theme.of(context).textTheme.button?.copyWith(color: Theme.of(context).accentColor),
                  ),
                  onPressed: onClick,
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }

  IconData _getIconForType(CustomAlertDialogType type) {
    switch (type) {
      case CustomAlertDialogType.warning:
        return Icons.warning;
      case CustomAlertDialogType.success:
        return Icons.check_circle;
      case CustomAlertDialogType.error:
        return Icons.error;
      case CustomAlertDialogType.info:
      default:
        return Icons.info_outline;
    }
  }

  static Color getColorForType(CustomAlertDialogType type) {
    switch (type) {
      case CustomAlertDialogType.warning:
        return Colors.orange;
      case CustomAlertDialogType.success:
        return Colors.green;
      case CustomAlertDialogType.error:
        return Colors.red;
      case CustomAlertDialogType.info:
      default:
        return Colors.blue;
    }
  }
}
