import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';

class GroupedItemInfo {
  int groupIndex;
  int itemIndex;
  GroupedItemInfo(this.groupIndex, this.itemIndex);
}

typedef GroupedListViewWidgetBuilder = Widget Function(BuildContext context, int index, GroupedItemInfo groupedItemInfo);

class GroupedListView<K, V extends Iterable> extends StatelessWidget {
  late final Map<K, V> items;
  final Axis scrollDirection;
  final bool reverse;
  final ScrollController? controller;
  final bool? primary;
  final ScrollPhysics? physics;
  final bool shrinkWrap;
  final EdgeInsetsGeometry? padding;
  final GroupedListViewWidgetBuilder itemBuilder;
  final GroupedListViewWidgetBuilder groupBuilder;
  final GroupedListViewWidgetBuilder separatorBuilder;
  final bool addAutomaticKeepAlives;
  final bool addRepaintBoundaries;
  final bool addSemanticIndexes;
  final double? cacheExtent;
  final DragStartBehavior dragStartBehavior;
  final ScrollViewKeyboardDismissBehavior keyboardDismissBehavior;
  final String? restorationId;
  final Clip clipBehavior;

  late final GroupedItemInfo Function(int index) itemInfoProvider;

  late final int itemCount;

  GroupedListView({
    Key? key,
    required Map<K, V> groupedItems,
    this.scrollDirection = Axis.vertical,
    this.reverse = false,
    this.controller,
    this.primary,
    this.physics,
    this.shrinkWrap = false,
    this.padding,
    required this.itemBuilder,
    required this.groupBuilder,
    required this.separatorBuilder,
    this.addAutomaticKeepAlives = true,
    this.addRepaintBoundaries = true,
    this.addSemanticIndexes = true,
    this.cacheExtent,
    this.dragStartBehavior = DragStartBehavior.start,
    this.keyboardDismissBehavior = ScrollViewKeyboardDismissBehavior.manual,
    this.restorationId,
    this.clipBehavior = Clip.hardEdge,
    bool preCalculatedGroupedItemInfo = true,
  }) : super(key: key) {
    itemCount = groupedItems.entries.length +
        groupedItems.entries.map((e) => e.value.length).reduce((value, element) => value + element);
    this.items = groupedItems;

    if (preCalculatedGroupedItemInfo) {
      final calculatedGroupItemInfos = _calculateGroupItemInfos();
      itemInfoProvider = (index) => calculatedGroupItemInfos[index];
    } else {
      itemInfoProvider = _getGroupedItemInfo;
    }
  }

  @override
  Widget build(BuildContext context) {
    return ListView.separated(
      itemBuilder: _itemBuilder,
      separatorBuilder: _separatorBuilder,
      itemCount: itemCount,
      scrollDirection: scrollDirection,
      reverse: reverse,
      controller: controller,
      primary: primary,
      physics: physics,
      shrinkWrap: shrinkWrap,
      padding: padding,
      cacheExtent: cacheExtent,
      dragStartBehavior: dragStartBehavior,
      keyboardDismissBehavior: keyboardDismissBehavior,
      restorationId: restorationId,
      clipBehavior: clipBehavior,
    );
  }

  Widget _itemBuilder(BuildContext context, int index) {
    var itemInfo = itemInfoProvider(index);
    if (itemInfo.itemIndex < 0) {
      return groupBuilder(context, index, itemInfo);
    } else {
      return itemBuilder(context, index, itemInfo);
    }
  }

  GroupedItemInfo _getGroupedItemInfo(int index) {
    var groupIndex = 0;
    var itemIndex = 0;

    while (true) {
      final groupItems = items.entries.elementAt(groupIndex).value;
      if (index < (itemIndex + groupItems.length + 1)) {
        return GroupedItemInfo(groupIndex, (index - itemIndex - 1));
      } else {
        groupIndex++;
      }
      // Store skipped items count + group item
      itemIndex += groupItems.length + 1;
    }
  }

  Widget _separatorBuilder(BuildContext context, int index) {
    var itemInfo = itemInfoProvider(index);
    return separatorBuilder(context, index, itemInfo);
  }

  List<GroupedItemInfo> _calculateGroupItemInfos() {
    var groupIndex = -1;

    return items.entries.map((e) {
      var itemIndex = 0;
      groupIndex++;
      return [GroupedItemInfo(groupIndex, -1), ...e.value.map((e) => GroupedItemInfo(groupIndex, itemIndex++))];
    }).reduce((value, element) => [...value, ...element]);
  }
}
