import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/scheduler.dart';

typedef ScrollWidgetBuilder = Widget Function(BuildContext context, ScrollController scrollController);

/// A Widget to load data on demand within scrollable Widgets
class InfiniteScrollFuture extends StatefulWidget {
  /// The [scrollController] is required to get the current scroll offset and calculate when to reload additional data;
  final ScrollController? scrollController;

  /// [onReloadRequest] is called when _scrollController.position.pixels > [positionToReload] * _scrollController.position.maxScrollExtent
  final AsyncCallback onReloadRequest;

  final ScrollWidgetBuilder builder;

  /// Defines the position to call [onReloadRequest] in percentage.
  final double positionToReload;

  const InfiniteScrollFuture({
    Key? key,
    required this.onReloadRequest,
    required this.builder,
    this.scrollController,
    this.positionToReload = 0.8,
  }) : super(key: key);

  @override
  State<InfiniteScrollFuture> createState() => _InfiniteScrollFutureState();
}

class _InfiniteScrollFutureState extends State<InfiniteScrollFuture> {
  late final ScrollController _scrollController;
  late double _reloadPosition;
  late double _lastNotifiedReloadPosition;
  late bool isLoading;


  @override
  void initState() {
    super.initState();
    _reloadPosition = 0;
    _lastNotifiedReloadPosition = 0;
    
    isLoading = false;
    _scrollController = widget.scrollController ?? ScrollController();
    _scrollController.addListener(_onScrollUpdate);
  }

  @override
  void dispose() {
    _scrollController.removeListener(_onScrollUpdate);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {

    SchedulerBinding.instance?.addPostFrameCallback(
      (_) {
        final pageHeight = MediaQuery.of(context).size.height;
        _reloadPosition = _scrollController.position.maxScrollExtent - (pageHeight/2);
      },
    );

    return widget.builder(context, _scrollController);
  }

  /// Check the current scroll position and request a reload if reolad position is reached.
  Future<void> _onScrollUpdate() async {
    if (isLoading) {
      return;
    }

    if (_scrollController.position.pixels > _reloadPosition) {
      try {
        if (mounted && _lastNotifiedReloadPosition != _reloadPosition) {
          isLoading = true;
          await widget.onReloadRequest();
          _lastNotifiedReloadPosition = _reloadPosition;
        }
      } catch (e) {
        throw e;
      } finally {
        isLoading = false;
      }
    }
  }
}
